"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpSubscriber = void 0;
const log_1 = require("../log");
var url = require('url');
const axios = require('axios');
class HttpSubscriber {
    constructor(express, options, io) {
        this.express = express;
        this.options = options;
        this.io = io;
    }
    subscribe(callback) {
        return new Promise((resolve, reject) => {
            this.express.post('/apps/:appId/events', (req, res) => {
                let body = [];
                res.on('error', (error) => {
                    if (this.options.devMode) {
                        log_1.Log.error(error);
                    }
                });
                req.on('data', (chunk) => body.push(chunk))
                    .on('end', () => this.handleData(req, res, body, callback));
            });
            log_1.Log.success('Listening for http events...');
            resolve();
        });
    }
    unsubscribe() {
        return new Promise((resolve, reject) => {
            try {
                this.express.post('/apps/:appId/events', (req, res) => {
                    res.status(404).send();
                });
                resolve();
            }
            catch (e) {
                reject('Could not overwrite the event endpoint -> ' + e);
            }
        });
    }
    handleData(req, res, body, broadcast) {
        body = JSON.parse(Buffer.concat(body).toString());
        if ((body.channels || body.channel) && body.name && body.data) {
            var data = body.data;
            try {
                data = JSON.parse(data);
            }
            catch (e) { }
            var message = {
                event: body.name,
                data: data,
                socket: body.socket_id
            };
            var channels = body.channels || [body.channel];
            if (this.options.devMode) {
                log_1.Log.info("Channel: " + channels.join(', '));
                log_1.Log.info("Event: " + message.event);
            }
            channels.forEach((channel) => {
                this.io.to(channel).emit(message.event, message.data);
                broadcast(channel, message);
            });
        }
        else {
            return this.badResponse(req, res, 'Event must include channel, event name and data');
        }
        res.json({ message: 'ok' });
        return true;
    }
    badResponse(req, res, message) {
        res.statusCode = 400;
        res.json({ error: message });
        return false;
    }
}
exports.HttpSubscriber = HttpSubscriber;
