"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisSubscriber = void 0;
var Redis = require('ioredis');
const log_1 = require("../log");
class RedisSubscriber {
    constructor(options) {
        this.options = options;
        this._keyPrefix = options.databaseConfig.redis.keyPrefix || '';
        this._redis = new Redis(options.databaseConfig.redis);
    }
    subscribe(callback) {
        return new Promise((resolve, reject) => {
            this._redis.on('pmessage', (subscribed, channel, message) => {
                try {
                    message = JSON.parse(message);
                    if (this.options.devMode) {
                        log_1.Log.info("Channel: " + channel);
                        log_1.Log.info("Event: " + message.event);
                    }
                    callback(channel.substring(this._keyPrefix.length), message);
                }
                catch (e) {
                    if (this.options.devMode) {
                        log_1.Log.info("No JSON message");
                    }
                }
            });
            this._redis.psubscribe(`${this._keyPrefix}*`, (err, count) => {
                if (err) {
                    reject('Redis could not subscribe.');
                }
                log_1.Log.success('Listening for redis events...');
                resolve();
            });
        });
    }
    unsubscribe() {
        return new Promise((resolve, reject) => {
            try {
                this._redis.disconnect();
                resolve();
            }
            catch (e) {
                reject('Could not disconnect from redis -> ' + e);
            }
        });
    }
}
exports.RedisSubscriber = RedisSubscriber;
